# this is a test

puts 1
puts 2
puts 3
puts 4
puts 5
puts 6
puts 7
puts 8
puts 9
puts 10
puts 11
puts 12
puts 13
puts 14
puts 15
puts 16
puts 17
puts 18
puts 19
puts 20
puts 21
puts 22
puts 23
puts 24
puts 25
puts 26
puts 27
puts 28
puts 29
puts 30
puts 31
puts 32
puts 33
puts 34
puts 35
puts 36
puts 37
puts 38
puts 39
puts 40
puts 41
puts 42
puts 43
puts 44
puts 45
puts 46
puts 47
puts 48
puts 49
puts 50
puts 51
puts 52
puts 53
puts 54
puts 55
puts 56
puts 57
puts 58
puts 59
puts 60
puts 61
puts 62
puts 63
puts 64
puts 65
puts 66
puts 67
puts 68
puts 69
puts 70
puts 71
puts 72
puts 73
puts 74
puts 75
puts 76
puts 77
puts 78
puts 79
puts 80
puts 81
puts 82
puts 83
puts 84
puts 85
puts 86
puts 87
puts 88
puts 89
puts 90
puts 91
puts 92
puts 93
puts 94
puts 95
puts 96
puts 97
puts 98
puts 99
puts 100
puts 101
puts 102
puts 103
puts 104
puts 105
puts 106
puts 107
puts 108
puts 109
puts 110
puts 111
puts 112
puts 113
puts 114
puts 115
puts 116
puts 117
puts 118
puts 119
puts 120
puts 121
puts 122
puts 123
puts 124
puts 125
puts 126
puts 127
puts 128
puts 129
puts 130
puts 131
puts 132
puts 133
puts 134
puts 135
puts 136
puts 137
puts 138
puts 139
puts 140
puts 141
puts 142
puts 143
puts 144
puts 145
puts 146
puts 147
puts 148
puts 149
puts 150
puts 151
puts 152
puts 153
puts 154
puts 155
puts 156
puts 157
puts 158
puts 159
puts 160
puts 161
puts 162
puts 163
puts 164
puts 165
puts 166
puts 167
puts 168
puts 169
puts 170
puts 171
puts 172
puts 173
puts 174
puts 175
puts 176
puts 177
puts 178
puts 179
puts 180
puts 181
puts 182
puts 183
puts 184
puts 185
puts 186
puts 187
puts 188
puts 189
puts 190
puts 191
puts 192
puts 193
puts 194
puts 195
puts 196
puts 197
puts 198
puts 199
puts 200
puts 201
puts 202
puts 203
puts 204
puts 205
puts 206
puts 207
puts 208
puts 209
puts 210
puts 211
puts 212
puts 213
puts 214
puts 215
puts 216
puts 217
puts 218
puts 219
puts 220
puts 221
puts 222
puts 223
puts 224
puts 225
puts 226
puts 227
puts 228
puts 229
puts 230
puts 231
puts 232
puts 233
puts 234
puts 235
puts 236
puts 237
puts 238
puts 239
puts 240
puts 241
puts 242
puts 243
puts 244
puts 245
puts 246
puts 247
puts 248
puts 249
puts 250
puts 251
puts 252
puts 253
puts 254
puts 255
puts 256
puts 257
puts 258
puts 259
puts 260
puts 261
puts 262
puts 263
puts 264
puts 265
puts 266
puts 267
puts 268
puts 269
puts 270
puts 271
puts 272
puts 273
puts 274
puts 275
puts 276
puts 277
puts 278
puts 279
puts 280
puts 281
puts 282
puts 283
puts 284
puts 285
puts 286
puts 287
puts 288
puts 289
puts 290
puts 291
puts 292
puts 293
puts 294
puts 295
puts 296
puts 297
puts 298
puts 299
puts 300
puts 301
puts 302
puts 303
puts 304
puts 305
puts 306
puts 307
puts 308
puts 309
puts 310
puts 311
puts 312
puts 313
puts 314
puts 315
puts 316
puts 317
puts 318
puts 319
puts 320
puts 321
puts 322
puts 323
puts 324
puts 325
puts 326
puts 327
puts 328
puts 329
puts 330
puts 331
puts 332
puts 333
puts 334
puts 335
puts 336
puts 337
puts 338
puts 339
puts 340
puts 341
puts 342
puts 343
puts 344
puts 345
puts 346
puts 347
puts 348
puts 349
puts 350
puts 351
puts 352
puts 353
puts 354
puts 355
puts 356
puts 357
puts 358
puts 359
puts 360
puts 361
puts 362
puts 363
puts 364
puts 365
puts 366
puts 367
puts 368
puts 369
puts 370
puts 371
puts 372
puts 373
puts 374
puts 375
puts 376
puts 377
puts 378
puts 379
puts 380
puts 381
puts 382
puts 383
puts 384
puts 385
puts 386
puts 387
puts 388
puts 389
puts 390
puts 391
puts 392
puts 393
puts 394
puts 395
puts 396
puts 397
puts 398
puts 399
puts 400
puts 401
puts 402
puts 403
puts 404
puts 405
puts 406
puts 407
puts 408
puts 409
puts 410
puts 411
puts 412
puts 413
puts 414
puts 415
puts 416
puts 417
puts 418
puts 419
puts 420
puts 421
puts 422
puts 423
puts 424
puts 425
puts 426
puts 427
puts 428
puts 429
puts 430
puts 431
puts 432
puts 433
puts 434
puts 435
puts 436
puts 437
puts 438
puts 439
puts 440
puts 441
puts 442
puts 443
puts 444
puts 445
puts 446
puts 447
puts 448
puts 449
puts 450
puts 451
puts 452
puts 453
puts 454
puts 455
puts 456
puts 457
puts 458
puts 459
puts 460
puts 461
puts 462
puts 463
puts 464
puts 465
puts 466
puts 467
puts 468
puts 469
puts 470
puts 471
puts 472
puts 473
puts 474
puts 475
puts 476
puts 477
puts 478
puts 479
puts 480
puts 481
puts 482
puts 483
puts 484
puts 485
puts 486
puts 487
puts 488
puts 489
puts 490
puts 491
puts 492
puts 493
puts 494
puts 495
puts 496
puts 497
puts 498
puts 499
puts 500
puts 501
puts 502
puts 503
puts 504
puts 505
puts 506
puts 507
puts 508
puts 509
puts 510
puts 511
puts 512
puts 513
puts 514
puts 515
puts 516
puts 517
puts 518
puts 519
puts 520
puts 521
puts 522
puts 523
puts 524
puts 525
puts 526
puts 527
puts 528
puts 529
puts 530
puts 531
puts 532
puts 533
puts 534
puts 535
puts 536
puts 537
puts 538
puts 539
puts 540
puts 541
puts 542
puts 543
puts 544
puts 545
puts 546
puts 547
puts 548
puts 549
puts 550
puts 551
puts 552
puts 553
puts 554
puts 555
puts 556
puts 557
puts 558
puts 559
puts 560
puts 561
puts 562
puts 563
puts 564
puts 565
puts 566
puts 567
puts 568
puts 569
puts 570
puts 571
puts 572
puts 573
puts 574
puts 575
puts 576
puts 577
puts 578
puts 579
puts 580
puts 581
puts 582
puts 583
puts 584
puts 585
puts 586
puts 587
puts 588
puts 589
puts 590
puts 591
puts 592
puts 593
puts 594
puts 595
puts 596
puts 597
puts 598
puts 599
puts 600
puts 601
puts 602
puts 603
puts 604
puts 605
puts 606
puts 607
puts 608
puts 609
puts 610
puts 611
puts 612
puts 613
puts 614
puts 615
puts 616
puts 617
puts 618
puts 619
puts 620
puts 621
puts 622
puts 623
puts 624
puts 625
puts 626
puts 627
puts 628
puts 629
puts 630
puts 631
puts 632
puts 633
puts 634
puts 635
puts 636
puts 637
puts 638
puts 639
puts 640
puts 641
puts 642
puts 643
puts 644
puts 645
puts 646
puts 647
puts 648
puts 649
puts 650
puts 651
puts 652
puts 653
puts 654
puts 655
puts 656
puts 657
puts 658
puts 659
puts 660
puts 661
puts 662
puts 663
puts 664
puts 665
puts 666
puts 667
puts 668
puts 669
puts 670
puts 671
puts 672
puts 673
puts 674
puts 675
puts 676
puts 677
puts 678
puts 679
puts 680
puts 681
puts 682
puts 683
puts 684
puts 685
puts 686
puts 687
puts 688
puts 689
puts 690
puts 691
puts 692
puts 693
puts 694
puts 695
puts 696
puts 697
puts 698
puts 699
puts 700
puts 701
puts 702
puts 703
puts 704
puts 705
puts 706
puts 707
puts 708
puts 709
puts 710
puts 711
puts 712
puts 713
puts 714
puts 715
puts 716
puts 717
puts 718
puts 719
puts 720
puts 721
puts 722
puts 723
puts 724
puts 725
puts 726
puts 727
puts 728
puts 729
puts 730
puts 731
puts 732
puts 733
puts 734
puts 735
puts 736
puts 737
puts 738
puts 739
puts 740
puts 741
puts 742
puts 743
puts 744
puts 745
puts 746
puts 747
puts 748
puts 749
puts 750
puts 751
puts 752
puts 753
puts 754
puts 755
puts 756
puts 757
puts 758
puts 759
puts 760
puts 761
puts 762
puts 763
puts 764
puts 765
puts 766
puts 767
puts 768
puts 769
puts 770
puts 771
puts 772
puts 773
puts 774
puts 775
puts 776
puts 777
puts 778
puts 779
puts 780
puts 781
puts 782
puts 783
puts 784
puts 785
puts 786
puts 787
puts 788
puts 789
puts 790
puts 791
puts 792
puts 793
puts 794
puts 795
puts 796
puts 797
puts 798
puts 799
puts 800
puts 801
puts 802
puts 803
puts 804
puts 805
puts 806
puts 807
puts 808
puts 809
puts 810
puts 811
puts 812
puts 813
puts 814
puts 815
puts 816
puts 817
puts 818
puts 819
puts 820
puts 821
puts 822
puts 823
puts 824
puts 825
puts 826
puts 827
puts 828
puts 829
puts 830
puts 831
puts 832
puts 833
puts 834
puts 835
puts 836
puts 837
puts 838
puts 839
puts 840
puts 841
puts 842
puts 843
puts 844
puts 845
puts 846
puts 847
puts 848
puts 849
puts 850
puts 851
puts 852
puts 853
puts 854
puts 855
puts 856
puts 857
puts 858
puts 859
puts 860
puts 861
puts 862
puts 863
puts 864
puts 865
puts 866
puts 867
puts 868
puts 869
puts 870
puts 871
puts 872
puts 873
puts 874
puts 875
puts 876
puts 877
puts 878
puts 879
puts 880
puts 881
puts 882
puts 883
puts 884
puts 885
puts 886
puts 887
puts 888
puts 889
puts 890
puts 891
puts 892
puts 893
puts 894
puts 895
puts 896
puts 897
puts 898
puts 899
puts 900
puts 901
puts 902
puts 903
puts 904
puts 905
puts 906
puts 907
puts 908
puts 909
puts 910
puts 911
puts 912
puts 913
puts 914
puts 915
puts 916
puts 917
puts 918
puts 919
puts 920
puts 921
puts 922
puts 923
puts 924
puts 925
puts 926
puts 927
puts 928
puts 929
puts 930
puts 931
puts 932
puts 933
puts 934
puts 935
puts 936
puts 937
puts 938
puts 939
puts 940
puts 941
puts 942
puts 943
puts 944
puts 945
puts 946
puts 947
puts 948
puts 949
puts 950
puts 951
puts 952
puts 953
puts 954
puts 955
puts 956
puts 957
puts 958
puts 959
puts 960
puts 961
puts 962
puts 963
puts 964
puts 965
puts 966
puts 967
puts 968
puts 969
puts 970
puts 971
puts 972
puts 973
puts 974
puts 975
puts 976
puts 977
puts 978
puts 979
puts 980
puts 981
puts 982
puts 983
puts 984
puts 985
puts 986
puts 987
puts 988
puts 989
puts 990
puts 991
puts 992
puts 993
puts 994
puts 995
puts 996
puts 997
puts 998
puts 999
puts 1000
